use std::io::{self, stdin};

use crate::blackjack::{BlackjackGame, GameState, PlayMoves};

pub fn play() -> Result<(), Box<dyn std::error::Error>> {
    let mut game = BlackjackGame::new();
    let mut play_move = PlayMoves::Bet;

    while !matches!(game.get_state(), GameState::Over) {
        match game.play(play_move) {
            crate::blackjack::PlayResult::DealerBlackJack => {
                print_full_state(&game);
                println!("Dealer wins with blackjack!");
            }
            crate::blackjack::PlayResult::PlayerBlackJack => {
                print_full_state(&game);
                println!("Player has a blackjack!");
            },
            crate::blackjack::PlayResult::PushBlackjack => {
                print_full_state(&game);
                println!("Player and Dealer have a blackjack!");
            }
            crate::blackjack::PlayResult::DealerBust => {
                print_full_state(&game);
                println!("Dealer busts. Player wins!");
            }
            crate::blackjack::PlayResult::StandPlayerLose => {
                print_full_state(&game);
                println!(
                    "Dealer wins with {}",
                    game.get_dealer_hand().get_blackjack_value()
                );
            }
            crate::blackjack::PlayResult::StandPlayerWin => {
                print_full_state(&game);
                println!(
                    "Player wins with {}",
                    game.get_player_hand().get_blackjack_value()
                );
            }
            crate::blackjack::PlayResult::Push => {
                print_full_state(&game);
                println!("Player and dealer have the same value. Push!");
            }
            crate::blackjack::PlayResult::Bust => {
                print_full_state(&game);
                println!(
                    "Player busts with {}",
                    game.get_player_hand().get_blackjack_value()
                );
            }
            crate::blackjack::PlayResult::InvalidMove => {
                println!("You cannot do that!");
                play_move = get_move()?;
            }
            crate::blackjack::PlayResult::Continue => {
                println!("Dealer: {} ?", game.get_dealer_hand().get_card(0).unwrap());
                println!(
                    "Player: {} ({})",
                    game.get_player_hand(),
                    game.get_player_hand().get_blackjack_value()
                );
                play_move = get_move()?
            }
        }
    }

    Ok(())
}

fn get_move() -> Result<PlayMoves, io::Error> {
    loop {
        println!("(H)it (S)tand (D)double");
        let mut buffer = String::new();

        stdin().read_line(&mut buffer)?;

        match buffer.trim() {
            "h" | "H" => return Ok(PlayMoves::Hit),
            "s" | "S" => return Ok(PlayMoves::Stand),
            "d" | "D" => return Ok(PlayMoves::DoubleDown),
            _ => {}
        }
    }
}

fn print_full_state(game: &BlackjackGame) {
    println!(
        "Dealer: {} ({})",
        game.get_dealer_hand(),
        game.get_dealer_hand().get_blackjack_value()
    );
    println!(
        "Player: {} ({})",
        game.get_player_hand(),
        game.get_player_hand().get_blackjack_value()
    );
}
